Package: multicool
Type: Package
Title: Permutations of Multisets in Cool-Lex Order
Version: 1.0.1
Date: 2024-02-05
Author: James Curran, Aaron Williams, Jerome Kelleher, Dave Barber
Maintainer: James Curran <j.curran@auckland.ac.nz>
Description: A set of tools to permute multisets without loops or hash tables and to generate integer partitions. The permutation functions are based on C code from Aaron Williams. Cool-lex order is similar to colexicographical order. The algorithm is described in Williams, A. Loopless Generation of Multiset Permutations by Prefix Shifts. SODA 2009, Symposium on Discrete Algorithms, New York, United States. The permutation code is distributed without restrictions. The code for stable and efficient computation of multinomial coefficients comes from Dave Barber. The code can be download from <http://tamivox.org/dave/multinomial/index.html> and is distributed without conditions. The package also generates the integer partitions of a positive, non-zero integer n. The C++ code for this is based on Python code from Jerome Kelleher which can be found here <https://jeromekelleher.net/category/combinatorics.html>. The C++ code and Python code are distributed without conditions.
URL: https://github.com/jmcurran/multicool
BugReports: https://github.com/jmcurran/multicool/issues
Encoding: UTF-8
License: GPL-2
Depends: methods, Rcpp (>= 0.11.2)
LinkingTo: Rcpp
RcppModules: Multicool
RoxygenNote: 7.2.3
NeedsCompilation: yes
Packaged: 2024-02-05 01:22:16 UTC; james
Repository: CRAN
Date/Publication: 2024-02-05 12:20:06 UTC
Built: R 4.3.3; aarch64-apple-darwin20; 2025-01-25 21:47:58 UTC; unix
Archs: multicool.so.dSYM

Package: multicool
Type: Package
Title: Permutations of Multisets in Cool-Lex Order
Version: 1.0.1
Date: 2024-02-05
Author: James Curran, Aaron Williams, Jerome Kelleher, Dave Barber
Maintainer: James Curran <j.curran@auckland.ac.nz>
Description: A set of tools to permute multisets without loops or hash tables and to generate integer partitions. The permutation functions are based on C code from Aaron Williams. Cool-lex order is similar to colexicographical order. The algorithm is described in Williams, A. Loopless Generation of Multiset Permutations by Prefix Shifts. SODA 2009, Symposium on Discrete Algorithms, New York, United States. The permutation code is distributed without restrictions. The code for stable and efficient computation of multinomial coefficients comes from Dave Barber. The code can be download from <http://tamivox.org/dave/multinomial/index.html> and is distributed without conditions. The package also generates the integer partitions of a positive, non-zero integer n. The C++ code for this is based on Python code from Jerome Kelleher which can be found here <https://jeromekelleher.net/category/combinatorics.html>. The C++ code and Python code are distributed without conditions.
URL: https://github.com/jmcurran/multicool
BugReports: https://github.com/jmcurran/multicool/issues
Encoding: UTF-8
License: GPL-2
Depends: methods, Rcpp (>= 0.11.2)
LinkingTo: Rcpp
RcppModules: Multicool
RoxygenNote: 7.2.3
NeedsCompilation: yes
Packaged: 2024-02-05 01:22:16 UTC; james
Repository: CRAN
Date/Publication: 2024-02-05 12:20:06 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 09:13:35 UTC; unix
Archs: multicool.so.dSYM

# Generated by roxygen2: do not edit by hand

export(B)
export(Bell)
export(S2)
export(Stirling2)
export(allPerm)
export(genComp)
export(initMC)
export(multinom)
export(nextPerm)
import(Rcpp)
import(methods)
useDynLib(multicool)

#' Calculate multinomial coefficients
#' 
#' This function calculates the number of permutations of a multiset, this
#' being the multinomial coefficient. If a set \eqn{X} contains \eqn{k} unique
#' elements \eqn{x_1, x_2, \ldots, x_k} with associate counts (or
#' multiplicities) of \eqn{n_1, n_2, \ldots, n_k}, then this function returns
#' \deqn{\frac{n!}{n_1!n_2!\ldots n_k!}}{n!/(n_1!n_2!\ldots n_k!)} where \eqn{n
#' = \sum_{i=1}{k}n_i}{n = n_1 + n_2 + \cdots + n_k}.
#' 
#' multinom depends on C++ code written by Dave Barber which can be found at
#' \url{http://tamivox.org/dave/multinomial/code.html}. The code may require
#' the STL algorithm library to be included in order to compile it.
#' 
#' @param x Either a multiset (with one or more potentially non-unique
#' elements), or if \code{counts} is \code{TRUE} a set of counts of the unique
#' elements of \eqn{X}. If \code{counts} is \code{FALSE} and \code{x} is not
#' numeric, then x will be coerced into an integer vector internally. If
#' \code{counts} is \code{TRUE} then \code{x} must be a vector of integers that
#' are greater than, or equal to zero.
#' @param counts if \code{counts} is TRUE, then this means x is the set of
#' counts \eqn{n_1, n_2, \ldots, n_k} rather than the set itself
#' @param useDouble if \code{useDouble} is TRUE then the computation will be
#' done using double precision floating point arithmetic. This option was added
#' because the internal code cannot handle integer overflow. The double
#' precision code will may a result that is closer to the truth for large
#' values, but this is not guaranteed. Ideally something like the GMP library
#' should be used, but this is not a priority at this point in time.
#' @return A single integer representing the multinomial coefficient for the
#' given multiset, or given set of multiplicities.
#' @author James M. Curran, Dave Barber
#' @references \url{http://tamivox.org/dave/multinomial/code.html}
#' @keywords combinations permutations multinomial
#' @examples
#' 
#' ## An example with a multiset X = (a,a,a,b,b,c)
#' ## There are 3 a s, 2 b s and 1 c, so the answer should be
#' ## (3+2+1)!/(3!2!1!) = 6!/3!2!1! = 60
#' x = rep(letters[1:3],3:1)
#' multinom(x)
#' 
#' ## in this example x is a vector of counts
#' ## the answer should be the same as above as x = c(3,2,1)
#' x = rep(letters[1:3],3:1)
#' x = as.vector(table(x)) #coerce x into a vector of counts
#' multinom(x, counts = TRUE)
#' 
#' 
#' ## An example of integer overflow. x is a vector of counts
#' ## c(12,11,8,8,6,5). The true answer from Maple is
#' ## 11,324,718,121,789,252,764,532,876,767,840,000
#' ## The error in the integer based answer is obvious.
#' ## The error using floating point is not, but from Maple is
#' ## 0.705057123232160000e+10
#' ## Thanks to Lev Dashevskiy for calling my attention to this.
#' \dontrun{x = c(12,11,8,8,6,5)
#' multinom(x, counts = TRUE, useDouble = FALSE)
#' multinom(x, counts = TRUE, useDouble = TRUE)
#' }
#' 
#' @export multinom
multinom = function(x, counts = FALSE, useDouble = FALSE){
  
  u = NULL
  
  if(!counts){
    u = as.vector(table(x))
  }else{
    ## make sure x is a vector of counts
    is.wholenumber = function(x, tol = .Machine$double.eps^0.5){abs(x - round(x)) < tol}
    
    if(any(!is.wholenumber(x)) || any(x < 0))
      stop("if counts == TRUE then all elements of x must be integer and >= 0")
    
    u = x
  }
  

  r = .Call('_multicool_multinomCoeff', PACKAGE = 'multicool', u, useDouble)
  
  return(r)
}

#' Generate all, or a subset, of the integer partitions of an integer n.
#' 
#' This function will return either all, or a length restricted subset of the
#' integer partitions of an integer n. The method works by considering
#' compositions rather than partions, hence the name.
#' 
#' This function will return all partions, or a subset, of an integer n. It
#' makes no check to see if this is a sensible thing to do. It also does it in
#' a lazy way in that in the restricted case it generates all partitions and
#' then only returns those that satistfy the length constraint. Users are
#' advised to check how many partitions are possible using partition number
#' function which is implemented the \code{P} function in the \pkg{partions}
#' package. Having said this P(50) is approximately 200 thousand, and P(100)
#' around 190 million, so the function should work well for smallish n.
#' 
#' @param n A positive non-zero integer
#' @param len Either logical \code{TRUE}, or an integer less than or equal to
#' \code{n}. If the latter form is used then only those partions of length less
#' than or equal to len are returned
#' @param addZeros If true then the empty partitions are added to the list of
#' partitions.
#' @return A list with each list element representing an integer partition
#' @note This function does not warn the user that the requested set of
#' partitions may be very large. In addition, all working is handled entirely
#' in memory, and so this may cause it to crash if the request is
#' execeptionally large.
#' @author Jerome Kelleher (algorithm and Python version) and James M. Curran
#' (C++ version/R interface)
#' @references Kelleher, J. (2005), Encoding Partitions As Ascending
#' Compositions, PhD thesis, University College Cork
#' \url{http://jeromekelleher.net/downloads/k06.pdf}.
#' 
#' Kelleher, J. and O'Sullivan, B. (2009), Generating All Partitions: A
#' Comparison Of Two Encodings, \url{http://arxiv.org/abs/0909.2331}.
#' 
#' Kelleher, J. (2010) Generating Integer
#' Partitions,\url{http://jeromekelleher.net/tag/integer-partitions.html}.
#' @keywords partitions
#' @examples
#' 
#' ## a small numeric example with all 11 partitions of 6
#' genComp(6)
#' 
#' ## a small example with the integer partitions of 6 of length 3 with empty partitions added
#' genComp(6, 3, TRUE)
#' 
#' ## a larger example - 627 partions of 20, but restricted to those of length 3 or smaller
#' genComp(20, 3)
#' 
#' @export genComp
genComp = function(n, len = TRUE, addZeros = FALSE){
  regularize = function(x, regLen){
    return(c(x, rep(0, regLen - length(x))))
  }
  if(len & is.logical(len)){
    
    l = generateCompositions(n)
    if(addZeros)
      l = lapply(l, regularize, regLen = n)
    return(l)
    
  }else if(is.numeric(len) & len <= n) {
    
    constraintFn = function(l){
      if(length(l) <= len)
        return(TRUE)
      else
        return(FALSE)
    }
    
    l = generateCompositions(n)
    l = l[sapply(l, constraintFn)]
    
    if(addZeros)
      l = lapply(l, regularize, regLen = len)
    
    return(l)
  }
}

#' Initialise the permutation object
#' 
#' This function initialises the permutation object. It must be called before
#' \code{nextPerm} can be called
#' 
#' 
#' @param x a vector of integers, reals, logicals or characters
#' @return a object of class \code{mc} which is a list containing elements
#' \itemize{ 
#' \item{mode}{ - the mode of the original data in \code{x}, "integer",
#' "double", or \code{mode(x)}}
#' \item{set}{ - either the multiset being permuted if \code{mode} is "integer" or
#' a set of integers corresponding to the elements of the multiset}
#' \item {elements}{ - if \code{mode} is not "integer" then this contains the
#' elements being permuted otherwise \code{NULL} }
#' \item {length} { - the length of the multiset }
#' \item {mc}{ - a pointer to the internal C++ Multicool object. Users
#' should not use this unless they really know what they are doing}
#' }
#' @author James M. Curran
#' @seealso nextPerm
#' @examples
#' 
#' x = c(1,1,2,2)
#' m1 = initMC(x)
#' m1
#' 
#' ## a non-integer example
#' 
#' x = rep(letters[1:4],c(2,1,2,2))
#' m2 = initMC(x)
#' m2
#' 
#' @export initMC
initMC = function(x){
  if(length(x) > 1){
    r = NULL

    if(is.numeric(x)){
      ## Test for whole number, with tolerance for representation
      ## From post by Tony Plate<tplate_at_acm.org>
      tolerance = .Machine$double.eps^0.5
      if(isTRUE(all(abs(x - round(x))<  tolerance))){ ## integer args
        tbl = table(x)
        elements = as.numeric(names(tbl))
        set = rep(1:length(tbl), tbl)
        
        r = new(Multicool, set)
        mcObj = list(mode = "integer", set = r$set(), elements = elements, 
                     length = r$length(), mc = r)
        
        class(mcObj) = "mc"
        return(mcObj)
      }else{ ## doubles
        tbl = table(x)
        elements = as.numeric(tbl)
        set = rep(1:length(tbl), tbl)
        
        r = new(Multicool, set)
        mcObj = list(mode = "double", set = r$set(), elements = elements,
                     length = r$length(), mc = r)
        class(mcObj) = "mc"
        return(mcObj)
      }
    }else{ ## logicals and characters and who know's whatelse
      tbl = table(x)
      elements = names(tbl)
      set = rep(1:length(tbl), tbl)
      
      r = new(Multicool, set)
      mcObj = list(mode = mode(x), set = r$set(), elements = elements,
                   length = r$length(), mc = r)
      class(mcObj) = "mc"
      return(mcObj)
    }
  }else{
    warning("The permutations of a vector of length 1 are not very interesting")
  }
}

#' Compute Stirling numbers of the second kind
#' 
#' This function computes Stirling numbers of the second kind, \eqn{S(n,
#' k)}{S(n, k)}, which count the number of ways of partitioning n distinct
#' objects in to k non-empty sets.
#' 
#' The implementation on this function is a simple recurrence relation which
#' defines \deqn{S(n, k) = kS(n - 1, k), + S(n - 1, k - 1)} for \eqn{k > 0}
#' with the inital conditions \eqn{S(0, 0) = 1} and \eqn{S(n, 0) = S(0, n) =
#' 0}. If \code{n} and \code{n} have different lengths then \code{expand.grid}
#' is used to construct a vector of (n, k) pairs
#' 
#' @aliases Stirling2 S2
#' @param n A vector of one or more positive integers
#' @param k A vector of one or more positive integers
#' @return An vector of Stirling numbers of the second kind
#' @author James Curran
#' @references
#' \url{http://en.wikipedia.org/wiki/Stirling_numbers_of_the_second_kind#Recurrence_relation}
#' @keywords partitions
#' @examples
#' 
#' ## returns S(6, 3)
#' Stirling2(6, 3)
#' 
#' ## returns S(6,1), S(6,2), ..., S(6,6)
#' S2(6, 1:6)
#' 
#' ## returns S(6,1), S(5, 2), S(4, 3)
#' S2(6:4, 1:3)
#' 
#' @export Stirling2
Stirling2 = function(n, k){
  if(any(n < 0) || any(k < 0))
    stop("n and k must be positive integers, n >= 0, k >= 0")
  
  nN = length(n)
  nK = length(k)
  
  if(nN > 1 || nK > 1){
    if(nN != nK){
      grid = expand.grid(n = n, k = k)
      n = grid$n
      k = grid$k
      
      nN = nK = nrow(grid)
    }
  }
  
  result = rep(0, nN)
  for(r in 1:nN){
    result[r] = Stirling2C(n[r], k[r])
  }
  
  return(result)
}

S2 = function(n, k){Stirling2(n, k)}



#' Compute the Bell numbers
#' 
#' This function computes the Bell numbers, which is the summ of Stirling
#' numbers of the second kind, \eqn{S(n, k)}{S(n, k)}, over \eqn{k = 1,\ldots,
#' n}{k}, i.e.  \deqn{B_n = \sum_{k=1}^{n}S(n, k),n \ge 1}
#' 
#' 
#' @aliases Bell B
#' @param n A vector of one or more non-zero positive integers
#' @return An vector of Bell numbers
#' @author James Curran
#' @seealso Stirling2
#' @references
#' \url{http://en.wikipedia.org/wiki/Stirling_numbers_of_the_second_kind#Recurrence_relation}
#' @keywords partitions
#' @examples
#' 
#' ## returns B(6)
#' Bell(6)
#' 
#' ## returns B(1), B(2), ..., B(6)
#' B(1:6)
#' 
#' @export Bell
Bell = function(n){
  if(any(n <=0))
    stop("n must be greater than or equal to 1")
  
  nN = length(n)
  result = rep(0, nN)
  
  for(r in 1:nN)
    result[r] = sum(Stirling2(n[r], 1:n[r]))
  
  return(result)
}

B = function(n){Bell(n)}

#' Generate and return all permutations of a multiset
#' 
#' This function will return all permutations of a multiset
#' 
#' This function will return all permutations of a multiset. It makes no check
#' to see if this is a sensible thing to do. Users are advised to check how
#' many permutations are possible using the \code{multinom} function in this
#' package.
#' 
#' @param mcObj an object of class mc - usually generated by \code{initMC}
#' @return A matrix with each row being a different permutation of the multiset
#' @note This function does not warn the user that the requested set of
#' permutations may be very large. In addition, all working is handled entirely
#' in memory, and so this may cause it to crash if the request is
#' execeptionally large.
#' @author James M. Curran
#' @seealso \code{\link{initMC}}, \code{\link{multinom}}
#' @keywords permutations
#' @examples
#' 
#' ## a small numeric example with 6 permuations
#' x = c(1,1,2,2)
#' m = initMC(x)
#' allPerm(m)
#' 
#' ## a large character example - 60 possibilities
#' x = rep(letters[1:3], 3:1)
#' multinom(x) ## calculate the number of permutations
#' m = initMC(x)
#' allPerm(m)
#' 
#' @export allPerm
allPerm = function(mcObj){
    if(class(mcObj) != "mc")
      stop("mcObject must be of class mc")
        
    r = mcObj$mc$allPerm()
    x = unlist(r)
    
    return(matrix(mcObj$elements[x], ncol = mcObj$length, byrow = TRUE))
}

#' Return the next permutation of the multiset
#' 
#' This function returns the next permuation of the multiset if there is one.
#' \code{initMC} called before \code{nextPerm} can be called.
#' 
#' 
#' @param mcObj an S3 object of class \code{mc} which must be created with
#' \code{initMC}
#' @return either a vector with the next permutation of the multiset or
#' \code{FALSE} when all permutations have been returned
#' @author James M. Curran
#' @seealso nextPerm
#' @examples
#' 
#' x = c(1,1,2,2)
#' m1 = initMC(x)
#' 
#' for(i in 1:6){
#'   cat(paste(paste(nextPerm(m1),collapse=","),"\n"))
#' }
#' 
#' ## an example with letters
#' x = letters[1:4]
#' m2 = initMC(x)
#' nextPerm(m2)
#' nextPerm(m2)
#' ## and so on
#' 
#' @export nextPerm
nextPerm = function(mcObj){
  r = mcObj$mc$nextPerm()
  
  if(r$b){
    return(mcObj$elements[r$set])
  }else{
    return(FALSE)
  }
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

generateCompositions <- function(n) {
    .Call('_multicool_generateCompositions', PACKAGE = 'multicool', n)
}

Stirling2C <- function(n, k) {
    .Call('_multicool_Stirling2C', PACKAGE = 'multicool', n, k)
}

BellC <- function(n) {
    .Call('_multicool_BellC', PACKAGE = 'multicool', n)
}

multinomCoeff <- function(x, useDouble) {
    .Call('_multicool_multinomCoeff', PACKAGE = 'multicool', x, useDouble)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Stirling2.R
\name{Stirling2}
\alias{Stirling2}
\alias{S2}
\title{Compute Stirling numbers of the second kind}
\usage{
Stirling2(n, k)
}
\arguments{
\item{n}{A vector of one or more positive integers}

\item{k}{A vector of one or more positive integers}
}
\value{
An vector of Stirling numbers of the second kind
}
\description{
This function computes Stirling numbers of the second kind, \eqn{S(n,
k)}{S(n, k)}, which count the number of ways of partitioning n distinct
objects in to k non-empty sets.
}
\details{
The implementation on this function is a simple recurrence relation which
defines \deqn{S(n, k) = kS(n - 1, k), + S(n - 1, k - 1)} for \eqn{k > 0}
with the inital conditions \eqn{S(0, 0) = 1} and \eqn{S(n, 0) = S(0, n) =
0}. If \code{n} and \code{n} have different lengths then \code{expand.grid}
is used to construct a vector of (n, k) pairs
}
\examples{

## returns S(6, 3)
Stirling2(6, 3)

## returns S(6,1), S(6,2), ..., S(6,6)
S2(6, 1:6)

## returns S(6,1), S(5, 2), S(4, 3)
S2(6:4, 1:3)

}
\references{
\url{http://en.wikipedia.org/wiki/Stirling_numbers_of_the_second_kind#Recurrence_relation}
}
\author{
James Curran
}
\keyword{partitions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multinom.R
\name{multinom}
\alias{multinom}
\title{Calculate multinomial coefficients}
\usage{
multinom(x, counts = FALSE, useDouble = FALSE)
}
\arguments{
\item{x}{Either a multiset (with one or more potentially non-unique
elements), or if \code{counts} is \code{TRUE} a set of counts of the unique
elements of \eqn{X}. If \code{counts} is \code{FALSE} and \code{x} is not
numeric, then x will be coerced into an integer vector internally. If
\code{counts} is \code{TRUE} then \code{x} must be a vector of integers that
are greater than, or equal to zero.}

\item{counts}{if \code{counts} is TRUE, then this means x is the set of
counts \eqn{n_1, n_2, \ldots, n_k} rather than the set itself}

\item{useDouble}{if \code{useDouble} is TRUE then the computation will be
done using double precision floating point arithmetic. This option was added
because the internal code cannot handle integer overflow. The double
precision code will may a result that is closer to the truth for large
values, but this is not guaranteed. Ideally something like the GMP library
should be used, but this is not a priority at this point in time.}
}
\value{
A single integer representing the multinomial coefficient for the
given multiset, or given set of multiplicities.
}
\description{
This function calculates the number of permutations of a multiset, this
being the multinomial coefficient. If a set \eqn{X} contains \eqn{k} unique
elements \eqn{x_1, x_2, \ldots, x_k} with associate counts (or
multiplicities) of \eqn{n_1, n_2, \ldots, n_k}, then this function returns
\deqn{\frac{n!}{n_1!n_2!\ldots n_k!}}{n!/(n_1!n_2!\ldots n_k!)} where \eqn{n
= \sum_{i=1}{k}n_i}{n = n_1 + n_2 + \cdots + n_k}.
}
\details{
multinom depends on C++ code written by Dave Barber which can be found at
\url{http://tamivox.org/dave/multinomial/code.html}. The code may require
the STL algorithm library to be included in order to compile it.
}
\examples{

## An example with a multiset X = (a,a,a,b,b,c)
## There are 3 a s, 2 b s and 1 c, so the answer should be
## (3+2+1)!/(3!2!1!) = 6!/3!2!1! = 60
x = rep(letters[1:3],3:1)
multinom(x)

## in this example x is a vector of counts
## the answer should be the same as above as x = c(3,2,1)
x = rep(letters[1:3],3:1)
x = as.vector(table(x)) #coerce x into a vector of counts
multinom(x, counts = TRUE)


## An example of integer overflow. x is a vector of counts
## c(12,11,8,8,6,5). The true answer from Maple is
## 11,324,718,121,789,252,764,532,876,767,840,000
## The error in the integer based answer is obvious.
## The error using floating point is not, but from Maple is
## 0.705057123232160000e+10
## Thanks to Lev Dashevskiy for calling my attention to this.
\dontrun{x = c(12,11,8,8,6,5)
multinom(x, counts = TRUE, useDouble = FALSE)
multinom(x, counts = TRUE, useDouble = TRUE)
}

}
\references{
\url{http://tamivox.org/dave/multinomial/code.html}
}
\author{
James M. Curran, Dave Barber
}
\keyword{combinations}
\keyword{multinomial}
\keyword{permutations}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/allPerm.R
\name{allPerm}
\alias{allPerm}
\title{Generate and return all permutations of a multiset}
\usage{
allPerm(mcObj)
}
\arguments{
\item{mcObj}{an object of class mc - usually generated by \code{initMC}}
}
\value{
A matrix with each row being a different permutation of the multiset
}
\description{
This function will return all permutations of a multiset
}
\details{
This function will return all permutations of a multiset. It makes no check
to see if this is a sensible thing to do. Users are advised to check how
many permutations are possible using the \code{multinom} function in this
package.
}
\note{
This function does not warn the user that the requested set of
permutations may be very large. In addition, all working is handled entirely
in memory, and so this may cause it to crash if the request is
execeptionally large.
}
\examples{

## a small numeric example with 6 permuations
x = c(1,1,2,2)
m = initMC(x)
allPerm(m)

## a large character example - 60 possibilities
x = rep(letters[1:3], 3:1)
multinom(x) ## calculate the number of permutations
m = initMC(x)
allPerm(m)

}
\seealso{
\code{\link{initMC}}, \code{\link{multinom}}
}
\author{
James M. Curran
}
\keyword{permutations}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/initMC.r
\name{initMC}
\alias{initMC}
\title{Initialise the permutation object}
\usage{
initMC(x)
}
\arguments{
\item{x}{a vector of integers, reals, logicals or characters}
}
\value{
a object of class \code{mc} which is a list containing elements
\itemize{ 
\item{mode}{ - the mode of the original data in \code{x}, "integer",
"double", or \code{mode(x)}}
\item{set}{ - either the multiset being permuted if \code{mode} is "integer" or
a set of integers corresponding to the elements of the multiset}
\item {elements}{ - if \code{mode} is not "integer" then this contains the
elements being permuted otherwise \code{NULL} }
\item {length} { - the length of the multiset }
\item {mc}{ - a pointer to the internal C++ Multicool object. Users
should not use this unless they really know what they are doing}
}
}
\description{
This function initialises the permutation object. It must be called before
\code{nextPerm} can be called
}
\examples{

x = c(1,1,2,2)
m1 = initMC(x)
m1

## a non-integer example

x = rep(letters[1:4],c(2,1,2,2))
m2 = initMC(x)
m2

}
\seealso{
nextPerm
}
\author{
James M. Curran
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genComp.R
\name{genComp}
\alias{genComp}
\title{Generate all, or a subset, of the integer partitions of an integer n.}
\usage{
genComp(n, len = TRUE, addZeros = FALSE)
}
\arguments{
\item{n}{A positive non-zero integer}

\item{len}{Either logical \code{TRUE}, or an integer less than or equal to
\code{n}. If the latter form is used then only those partions of length less
than or equal to len are returned}

\item{addZeros}{If true then the empty partitions are added to the list of
partitions.}
}
\value{
A list with each list element representing an integer partition
}
\description{
This function will return either all, or a length restricted subset of the
integer partitions of an integer n. The method works by considering
compositions rather than partions, hence the name.
}
\details{
This function will return all partions, or a subset, of an integer n. It
makes no check to see if this is a sensible thing to do. It also does it in
a lazy way in that in the restricted case it generates all partitions and
then only returns those that satistfy the length constraint. Users are
advised to check how many partitions are possible using partition number
function which is implemented the \code{P} function in the \pkg{partions}
package. Having said this P(50) is approximately 200 thousand, and P(100)
around 190 million, so the function should work well for smallish n.
}
\note{
This function does not warn the user that the requested set of
partitions may be very large. In addition, all working is handled entirely
in memory, and so this may cause it to crash if the request is
execeptionally large.
}
\examples{

## a small numeric example with all 11 partitions of 6
genComp(6)

## a small example with the integer partitions of 6 of length 3 with empty partitions added
genComp(6, 3, TRUE)

## a larger example - 627 partions of 20, but restricted to those of length 3 or smaller
genComp(20, 3)

}
\references{
Kelleher, J. (2005), Encoding Partitions As Ascending
Compositions, PhD thesis, University College Cork
\url{http://jeromekelleher.net/downloads/k06.pdf}.

Kelleher, J. and O'Sullivan, B. (2009), Generating All Partitions: A
Comparison Of Two Encodings, \url{http://arxiv.org/abs/0909.2331}.

Kelleher, J. (2010) Generating Integer
Partitions,\url{http://jeromekelleher.net/tag/integer-partitions.html}.
}
\author{
Jerome Kelleher (algorithm and Python version) and James M. Curran
(C++ version/R interface)
}
\keyword{partitions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Stirling2.R
\name{Bell}
\alias{Bell}
\alias{B}
\title{Compute the Bell numbers}
\usage{
Bell(n)
}
\arguments{
\item{n}{A vector of one or more non-zero positive integers}
}
\value{
An vector of Bell numbers
}
\description{
This function computes the Bell numbers, which is the summ of Stirling
numbers of the second kind, \eqn{S(n, k)}{S(n, k)}, over \eqn{k = 1,\ldots,
n}{k}, i.e.  \deqn{B_n = \sum_{k=1}^{n}S(n, k),n \ge 1}
}
\examples{

## returns B(6)
Bell(6)

## returns B(1), B(2), ..., B(6)
B(1:6)

}
\references{
\url{http://en.wikipedia.org/wiki/Stirling_numbers_of_the_second_kind#Recurrence_relation}
}
\seealso{
Stirling2
}
\author{
James Curran
}
\keyword{partitions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nextPerm.R
\name{nextPerm}
\alias{nextPerm}
\title{Return the next permutation of the multiset}
\usage{
nextPerm(mcObj)
}
\arguments{
\item{mcObj}{an S3 object of class \code{mc} which must be created with
\code{initMC}}
}
\value{
either a vector with the next permutation of the multiset or
\code{FALSE} when all permutations have been returned
}
\description{
This function returns the next permuation of the multiset if there is one.
\code{initMC} called before \code{nextPerm} can be called.
}
\examples{

x = c(1,1,2,2)
m1 = initMC(x)

for(i in 1:6){
  cat(paste(paste(nextPerm(m1),collapse=","),"\\n"))
}

## an example with letters
x = letters[1:4]
m2 = initMC(x)
nextPerm(m2)
nextPerm(m2)
## and so on

}
\seealso{
nextPerm
}
\author{
James M. Curran
}
